<?php
/**
 * Plugin Filehtml (addon for plugin Adm_redacteurs)
 * Version  : 1.1 (2016/05/01)
 * Compatibility : Guppy v5.0+ et Adm_redacteurs v6.8+
 * Author   : jérôme CROUX (jchouix)
 * Web site : http://lebrikabrak.info/
 * E-mail   : lebrikabrak@free.fr
 * Licence  : GNU Lesser General Public License
 */

defined('ADMR_DIRNAME') OR exit('No direct script access allowed');

class Filehtml_params_model extends Model
{    
    /*
    |--------------------------------------------------------------------------
    | Properties
    |--------------------------------------------------------------------------
    */
    private $_data_DB = array(); // Données de la base de données
    private $_file_dir_path = FILEHTML_PARAMS_PATH; // Chemin du répertoire contenant les fichiers des données des documents
    private $_file_prefix = FILEHTML_PARAMS_FILE_PREFIX; // Préfixe du nom du fichier contenant les données du document
    private $_file_ext = INCEXT;

    private $_params = array(
        'id' => 0, // Id du document
        'title1' => '', // Titre du document (langue 1)
        'title2' => '', // Titre du document (langue 2)
        'author_id' => 0, // Id de l'auteur du document (0 = admin)
        'group' => '', // Groupe privé de guppy
        'createdate' => '', // Dates de publication sous la forme date0('YmdHi') - date1('YmdHi')
        'create_datetime' => '', // Date de création (timestamp)
        'modif_datetime' => '',  // Date de modification (timestamp)      
        'link1' => '', // uri du fichier html (langue 1)
        'link2' => '', // uri du fichier html (langue 2)
        'iframe1' => '', // iframe (langue 1)
        'iframe2' => '', // iframe (langue 2)
        'iframe_height1' => '', // Hauteur de l'iframe (langue 1)
        'iframe_height2' => '', // Hauteur de l'iframe (langue 2)

        'contents_before1' => '', // contenu avant le fichier html (langue 1)
        'contents_before2' => '', // contenu avant le fichier html (langue 2)
        'contents_after1' => '', // contenu après le fichier html (langue 1)
        'contents_after2' => '', // contenu après le fichier html (langue 2)

        'previous_page_label1' => '', // Intitulé de la page précédente (langue 1)
        'previous_page_uri1' => '', // Lien qui mène vers la page précédente (langue 1)
        'next_page_label1' => '', // Intitulé de la page suivante (langue 1)
        'next_page_uri1' => '', // Lien qui mène vers la page suivante (langue 1)
        'previous_page_label2' => '', // Intitulé de la page précédente (langue 2)
        'previous_page_uri2' => '', // Lien qui mène vers la page précédente (langue 2)
        'next_page_label2' => '', // Intitulé de la page suivante (langue 2)
        'next_page_uri2' => '', // Lien qui mène vers la page suivante (langue 2)

        'display_author' => '', // Affiche le nom de l'auteur
        'display_create_date' => '', // Affiche la date de création
        'display_modif_date' => '', // Affiche la date de modification
        'display_nb_views' => '', // Affiche le nombre de vues
        
        'pubdate0'   => '', // Date début de publication programmée
        'pubhour0'   => '', // Heure de début de publication programmée
        'pubminute0' => '', // Minute de début de publication programmée
        'pubdate1'   => '', // Date de fin de publication programmée
        'pubhour1'   => '', // Heure de fin de publication programmée
        'pubminute1' => '', // Minute de fin de publication programmée
    );

    /*
    |--------------------------------------------------------------------------
    | Constructor
    |--------------------------------------------------------------------------
    */
    public function __construct()
	{

	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie le nom des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_all_files()
	{
        $files = array();
        
        if (!empty($this -> _file_dir_path)) {
            $files = array_slice(scandir($this -> _file_dir_path), 2);
            unset($files[array_search('index.php', $files)]); // Suppression du fichier index.php
        }
        
        return $files;
	}

	// --------------------------------------------------------------------

	/**
	 * Renvoie les identifiants des fichiers enregistrés dans la base de donnée
     * 
     * @access public 
	 * @return boolean
	 */
    public function get_files_ids()
	{
        $files_ids = array();
        $files = $this -> get_all_files();

        foreach($files as $key => $file) {
            $files_ids[] = str_replace(array($this -> _file_prefix, $this -> _file_ext), '', $file);
        }
        
        return $files_ids;
	}

	// --------------------------------------------------------------------

	/**
	 * Vérifie si le document existe ou non
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return boolean
	 */
    public function file_exists($doc_id = 0)
	{
        if (!empty($doc_id) && ctype_digit((string) $doc_id) && is_file($this -> _get_file_path($doc_id))) {            
            return TRUE;    
        }
        
        return FALSE;
	}

	// --------------------------------------------------------------------

	/**
	 * Lit et renvoie dans un tableau les données enregistrées dans le document identifié par la valeur de son id
     * 
     * @access public 
	 * @param integer Identifiant du document
	 * @return array Tableau contenant les valeurs du document
	 */
    public function read($doc_id = 0)
    {
        $data = array_merge($this -> _params,
            array(
                'pubdate0'   => Date_helper :: get_current_date(),   // Date début de publication programmée
                'pubhour0'   => '00',   // Heure de début de publication programmée
                'pubminute0' => '00',   // Minute de début de publication programmée
                'iframe1' => '1', // Iframe (langue 1)
                'iframe2' => '1', // Iframe (langue 2)
                'iframe_height1' => '800', // Hauteur de l'iframe (langue 1)
                'iframe_height2' => '800', // Hauteur de l'iframe (langue 2)
            )
        );
        
        if ($this -> file_exists($doc_id)) {
            $data = array_merge($data, (array) unserialize(file_get_contents($this -> _get_file_path($doc_id))));    
        }
       
        return $data; 
    }
    
	// --------------------------------------------------------------------
 
	/**
	 * Ecrit les données du document dans un fichier
     * 
     * @access public 
     * @param array Données du rédacteur ou du groupe rédacteur
	 * @return array
	 */    
    public function write($data)
    {
        settype($data, 'array');

        if (empty($data['id']) || ! ctype_digit((string) $data['id'])) {
            return FALSE;
        }
      
        $file_path = $this -> _get_file_path($data['id']);
        $data = $this -> _clean($data);
        $data['createdate'] = Date_helper :: convert_pubdates_to_createdate(array($data['pubdate0'], $data['pubdate1']), array($data['pubhour0'], $data['pubhour1']), array($data['pubminute0'], $data['pubminute1']));
		$data['create_datetime'] = Date_helper :: get_time_from_pubdate($data['pubdate0'], $data['pubhour0'], $data['pubminute0']);
        $data['modif_datetime']  = time();
                
		return file_put_contents($file_path, serialize($data), LOCK_EX);
    }

	// --------------------------------------------------------------------
 
	/**
	 * Supprime les documents identifiés par leur id
     * 
     * @access public 
     * @param array|integer Identifiants des documents à supprimer
	 * @return boolean
	 */    
    public function delete_by_id($doc_ids)
    {
        settype($doc_ids, 'array');

        if (empty($this -> _file_dir_path) || empty($doc_ids)) {
            return FALSE;
        }
                
        foreach($doc_ids as $doc_id) {
            if ($this -> file_exists($doc_id)) {
                @unlink($this -> _get_file_path($doc_id));    
            }
        }
       
        return TRUE; 
    }
   
	// --------------------------------------------------------------------
 
	/**
	 * Retourne le chemin du fichier associé à l'id passé en paramètre
     * 
     * @access private 
     * @param integer Id du document
	 * @return string Chemin du fichier associé à l'id passé en paramètre
	 */    
    private function _get_file_path($doc_id)
    {
        return $this -> _file_dir_path.'/'.$this -> _file_prefix.$doc_id.$this -> _file_ext;    
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données du rédacteur
     * 
     * @access private 
	 * @param array Données du rédacteur à nettoyer
	 * @return array
	 */        
    private function _clean($data)
    {
        $data_by_default = $this -> _params;
        $data = $this -> _parse_params_contents($data);
                
        return array_merge($data_by_default, array_intersect_key($data, $data_by_default));
    }

	// --------------------------------------------------------------------
 
	/**
	 * Nettoie les données de type "contenu"
     * 
     * @access private 
	 * @param array Données à nettoyer
	 * @return array
	 */
    private function _parse_params_contents($params)
    {
        $in = array(
    		'\\',
    		'="'.ADMR_SITE_URL
    	);

    	$out = array(
    		'&#92;',
    		'="'
    	);       
        
        foreach(array('contents_before1', 'contents_before2', 'contents_after1', 'contents_after2') as $key) {
            if (!empty($params[$key])) {
                $params[$key] = str_replace($in, $out, $params[$key]);
            }            
        }        

        return $params;
    }

	// --------------------------------------------------------------------
  
	/**
	 * Définit et renvoie les règles de validation des paramètres
     * 
     * @access public 
	 * @param array Tableau contenant les paramètres à valider
	 * @param array Tableau contenant les messages d'erreur
	 * @param array Tableau contenant les options de configuration
	 * @return array Tableau contenant les règles de validation et les messages d'erreur associés
	 */
    public function set_params_rules($params, $lang, $config)
    {
        return array_merge(
            Params_rules_helper :: set_params_status_rules($lang, (ADMR_IS_ADMIN) ? array('a', 'i', 'd') : array('a', 'i')), // Status
            Params_rules_helper :: set_params_pubdates_rules($params, $lang), // Dates de publication
            Params_rules_helper :: set_params_group_rules($lang),             // Group privé
            Params_rules_helper :: set_params_title_rules($lang, $config),    // Titre du document
            $this -> _set_params_links_rules($lang, $config), // Liens vers les fichiers html
            $this -> _set_params_iframe_height_rules($lang, $config), // Hauteur des iframes

            array(
   /*             'nb_questions' => array( // Nombre de questions du filehtml
                    array(
                        'rules' => array(array('Var_helper', 'is_between[1,'.$config['filehtml']['nb_questions_max'].']')),
                        'msg' => htmlspecialchars($lang['form_errors_msg']['field_not_valid'], ENT_QUOTES, 'UTF-8')
                    ),
                ),*/
            )
        );
    }

	// --------------------------------------------------------------------
 
	/**
	 * Définit et renvoie les règles de validation des paramètres link1 et link2
     * 
     * @access protected
	 * @param array Tableau contenant les messages d'erreur
	 * @param array Tableau contenant les options de configuration
	 * @return array Tableau contenant les règles de validation et les messages d'erreur associés
	 */
    protected function _set_params_links_rules($lang, $config)
    {
        $links = empty($config['lang'][1]) ? array('link1') : array('link1', 'link2'); 
        
        foreach($links as $link) {
            $rules[$link] = array(       // Lien vers le fichier html
                array(
                    'rules' => array(array('Var_helper', 'is_required')),
                    'msg' => htmlspecialchars($lang['form_errors_msg']['file_required'], ENT_QUOTES, 'UTF-8')
                ),
                array(
                    'rules' => array(array('Var_helper', 'is_in_directories['.implode(',', $config['file_manager']['access']['filehtml']['dirs']).']')),
                    'msg' => sprintf(htmlspecialchars($lang['form_errors_msg']['only_this_file_directories_%s_are_allowed'], ENT_QUOTES, 'UTF-8'), implode(',', $config['file_manager']['access']['filehtml']['dirs'])),
                ),
                array(
                    'rules' => array(array('Var_helper', 'is_allowed_ext['.implode(',', $config['file_manager']['access']['filehtml']['allowed_ext']).']')),
                    'msg' => sprintf(htmlspecialchars($lang['form_errors_msg']['only_this_file_extensions_%s_are_allowed'], ENT_QUOTES, 'UTF-8'), implode(',', $config['file_manager']['access']['filehtml']['allowed_ext'])),
                ),
            );
        }
        
        return $rules;
     }

	// --------------------------------------------------------------------
 
	/**
	 * Définit et renvoie les règles de validation des paramètres iframe_height1 et iframe_height2
     * 
     * @access protected
	 * @param array Tableau contenant les messages d'erreur
	 * @param array Tableau contenant les options de configuration
	 * @return array Tableau contenant les règles de validation et les messages d'erreur associés
	 */
    protected function _set_params_iframe_height_rules($lang, $config)
    {
        foreach(array('iframe_height1', 'iframe_height2') as $iframe_height) {
            $rules[$iframe_height] = array(       // Lien vers le fichier html
                array(
                    'rules' => array(array('Var_helper', 'is_digit_no_zero')),
                    'msg' => htmlspecialchars($lang['form_errors_msg']['field_not_number_valid'], ENT_QUOTES, 'UTF-8')
                ),
            );
        }
        
        return $rules;
     }
     
	// --------------------------------------------------------------------

} // End of Class